



if not exist %LIBRARY_INC%\apr.h             exit 1
IF %ERRORLEVEL% NEQ 0 exit 1
if not exist %LIBRARY_INC%\apu.h             exit 1
IF %ERRORLEVEL% NEQ 0 exit 1
if not exist %LIBRARY_BIN%\libapr-1.dll      exit 1
IF %ERRORLEVEL% NEQ 0 exit 1
if not exist %LIBRARY_BIN%\libaprutil-1.dll  exit 1
IF %ERRORLEVEL% NEQ 0 exit 1
if not exist %LIBRARY_BIN%\libapriconv-1.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit 1
exit 0
